#include <limits>

#include "Evaluation.hpp"

Score Evaluation::material(const Board& board) {
    Score result = 0;

    for (auto square: board.squares) {
        auto piece = square.second;
        if (piece.color == board.turn_color)
            result += Evaluation::piece(piece);
        else
            result -= Evaluation::piece(piece);
    }

    return result;
}

Score Evaluation::material_sum(const Board& board) {
    Score result = 0;

    for (auto square: board.squares) {
        auto piece = square.second;
        result += Evaluation::piece(piece);
    }

    return result;
}

Score Evaluation::positional(const Board& board) {
    return 0;
}

Score Evaluation::total(const Board& board) {
    return material(board) + positional(board);
}

// Based on http://home.comcast.net/~danheisman/Articles/evaluation_of_material_imbalance.htm
Score Evaluation::piece(const Piece& piece) {
    Score result =
        (piece.type == PAWN)?   1000:
        (piece.type == KNIGHT)? 3250:
        (piece.type == BISHOP)? 3250:
        (piece.type == ROOK)?   5000:
        (piece.type == QUEEN)?  9750:
        0; // for king

    return result;
}

const UNumber max_mate_halfmoves = 10000;

Score Evaluation::mate() {
    return std::numeric_limits<SNumber>::max();
}

bool Evaluation::is_mate(const Score& score) {
    if (score > 0)
        return mate() - score <= max_mate_halfmoves;
    else
        return (mate() + score >= 0) && (mate() + score <= 10000);
}

UNumber Evaluation::get_mate_half_moves(const Score& score) {
    if (score > 0)
        return mate() - score;
    else
        return (mate() + score);
}
