#include "Piece.hpp"
#include "Exceptions.hpp"

Piece::Piece()
    : color(NOT_A_COLOR), type(NOT_A_TYPE)
{
    // none
}

Piece::Piece(const Color& color, const PieceType& type)
    : color(color), type(type)
{
    // none
}

std::string Piece::to_fen() const {
    if (color == WHITE && type == PAWN) return "P";
    if (color == WHITE && type == KNIGHT) return "N";
    if (color == WHITE && type == BISHOP) return "B";
    if (color == WHITE && type == ROOK) return "R";
    if (color == WHITE && type == QUEEN) return "Q";
    if (color == WHITE && type == KING) return "K";

    if (color == BLACK && type == PAWN) return "p";
    if (color == BLACK && type == KNIGHT) return "n";
    if (color == BLACK && type == BISHOP) return "b";
    if (color == BLACK && type == ROOK) return "r";
    if (color == BLACK && type == QUEEN) return "q";
    if (color == BLACK && type == KING) return "k";

    throw CustomException("Bad piece in Piece::to_fen()");
}

Piece Piece::from_fen(std::string fen) {
    if (fen == "p") return Piece(BLACK, PAWN);
    if (fen == "n") return Piece(BLACK, KNIGHT);
    if (fen == "b") return Piece(BLACK, BISHOP);
    if (fen == "r") return Piece(BLACK, ROOK);
    if (fen == "q") return Piece(BLACK, QUEEN);
    if (fen == "k") return Piece(BLACK, KING);

    if (fen == "P") return Piece(WHITE, PAWN);
    if (fen == "N") return Piece(WHITE, KNIGHT);
    if (fen == "B") return Piece(WHITE, BISHOP);
    if (fen == "R") return Piece(WHITE, ROOK);
    if (fen == "Q") return Piece(WHITE, QUEEN);
    if (fen == "K") return Piece(WHITE, KING);

    throw CustomException("Bad fen in Piece::from_fen(). (" + fen + ")");
}

bool Piece::operator==(const Piece& rhs) const {
    return (*this).color == rhs.color && (*this).type == rhs.type;
}

bool Piece::operator!=(const Piece& rhs) const {
    return (*this).color != rhs.color || (*this).type != rhs.type;
}
