#include "RookingPosibility.hpp"
#include "Exceptions.hpp"
#include "Platform.hpp"

RookingPosibility::RookingPosibility()
    : RookingPosibility(false, false, false, false)
{
    // none
}

RookingPosibility::RookingPosibility(bool wk, bool wq, bool bk, bool bq)
    : white_king(wk), white_queen(wq), black_king(bk), black_queen(bq)
{
    // none
}

RookingPosibility RookingPosibility::from_fen(std::string str) {
    if (str == "-") return RookingPosibility(false, false, false, false);

    bool wk = (str.find('K') != std::string::npos);
    bool wq = (str.find('Q') != std::string::npos);
    bool bk = (str.find('k') != std::string::npos);
    bool bq = (str.find('q') != std::string::npos);

    const UNumber valid = wk+wq+bk+bq;

    if (valid != str.length())
        throw CustomException("Bad fen in RookingPosibility::from_fen()");

    return RookingPosibility(wk, wq, bk, bq);
}

RookingPosibility RookingPosibility::none() {
    return RookingPosibility(false, false, false, false);
}


RookingPosibility RookingPosibility::all() {
    return RookingPosibility(true, true, true, true);
}

std::string RookingPosibility::to_fen() const {
    if (*this == RookingPosibility(false, false, false, false)) return "-";

    std::string result = "";

    if ((*this).white_king)  result += 'K';
    if ((*this).white_queen) result += 'Q';
    if ((*this).black_king)  result += 'k';
    if ((*this).black_queen) result += 'q';

    return result;
}

bool RookingPosibility::operator==(const RookingPosibility& rhs) const {
    if ((*this).white_king  != rhs.white_king)  return false;
    if ((*this).white_queen != rhs.white_queen) return false;
    if ((*this).black_king  != rhs.black_king)  return false;
    if ((*this).black_queen != rhs.black_queen) return false;

    return true;
}

bool RookingPosibility::operator!=(const RookingPosibility& rhs) const {
    if ((*this).white_king  != rhs.white_king)  return true;
    if ((*this).white_queen != rhs.white_queen) return true;
    if ((*this).black_king  != rhs.black_king)  return true;
    if ((*this).black_queen != rhs.black_queen) return true;

    return false;
}
